#include "testing_flags.cuh"

#ifdef TEST_GROUP_WGMMA_MMA_FP16_FP16

#include "testing_commons.cuh"

namespace group {
namespace wgmma {
namespace mma_fp16_fp16 {
void tests(test_data &results);
}
}
}

#endif